@@ -20,18 +20,16 @@ struct _NautilusCompressDialog
     NautilusFilenameValidator *validator;
 
     GtkWidget *name_entry;
-    GtkWidget *extension_dropdown;
-    GtkSizeGroup *extension_sizegroup;
-    GtkWidget *passphrase_label;
+    AdwComboRow *extension_combo_row;
     GtkWidget *passphrase_entry;
-    GtkWidget *passphrase_confirm_label;
+    GtkWidget *passphrase_group;
     GtkWidget *passphrase_confirm_entry;
 
     CompressCallback callback;
     gpointer callback_data;
 };
 
-G_DEFINE_TYPE (NautilusCompressDialog, nautilus_compress_dialog, ADW_TYPE_WINDOW);
+G_DEFINE_TYPE (NautilusCompressDialog, nautilus_compress_dialog, ADW_TYPE_DIALOG);
 
 #define NAUTILUS_TYPE_COMPRESS_ITEM (nautilus_compress_item_get_type ())
 G_DECLARE_FINAL_TYPE (NautilusCompressItem, nautilus_compress_item, NAUTILUS, COMPRESS_ITEM, GObject)
@@ -40,6 +38,7 @@ struct _NautilusCompressItem
 {
     GObject parent_instance;
     NautilusCompressionFormat format;
+    gchar *title;
     char *extension;
     char *description;
 };
@@ -57,11 +56,18 @@ nautilus_compress_item_finalize (GObject *object)
     NautilusCompressItem *item = NAUTILUS_COMPRESS_ITEM (object);
 
     g_free (item->extension);
+    g_free (item->title);
     g_free (item->description);
 
     G_OBJECT_CLASS (nautilus_compress_item_parent_class)->finalize (object);
 }
 
+static char *
+nautilus_compress_item_dup_title (NautilusCompressItem *item)
+{
+    return g_strdup (item->title);
+}
+
 static void
 nautilus_compress_item_class_init (NautilusCompressItemClass *class)
 {
@@ -73,12 +79,14 @@ nautilus_compress_item_class_init (NautilusCompressItemClass *class)
 static NautilusCompressItem *
 nautilus_compress_item_new (NautilusCompressionFormat  format,
                             const char                *extension,
+                            const char                *title,
                             const char                *description)
 {
     NautilusCompressItem *item = g_object_new (NAUTILUS_TYPE_COMPRESS_ITEM, NULL);
 
     item->format = format;
     item->extension = g_strdup (extension);
+    item->title = g_strdup (title);
     item->description = g_strdup (description);
 
     return item;
@@ -88,10 +96,7 @@ static void
 set_show_passphrase (NautilusCompressDialog *self,
                      gboolean                show_passphrase)
 {
-    gtk_widget_set_visible (self->passphrase_label, show_passphrase);
-    gtk_widget_set_visible (self->passphrase_entry, show_passphrase);
-    gtk_widget_set_visible (self->passphrase_confirm_label, show_passphrase);
-    gtk_widget_set_visible (self->passphrase_confirm_entry, show_passphrase);
+    gtk_widget_set_sensitive (self->passphrase_group, show_passphrase);
     if (!show_passphrase)
     {
         gtk_editable_set_text (GTK_EDITABLE (self->passphrase_entry), "");
@@ -102,7 +107,7 @@ set_show_passphrase (NautilusCompressDialog *self,
 static void
 update_selected_format (NautilusCompressDialog *self)
 {
-    NautilusCompressItem *item = gtk_drop_down_get_selected_item (GTK_DROP_DOWN (self->extension_dropdown));
+    NautilusCompressItem *item = adw_combo_row_get_selected_item (self->extension_combo_row);
     if (item == NULL)
     {
         return;
@@ -121,41 +126,37 @@ update_selected_format (NautilusCompressDialog *self)
 }
 
 static void
-extension_dropdown_setup_item (GtkSignalListItemFactory *factory,
-                               GtkListItem              *item,
-                               gpointer                  user_data)
+on_feedback_changed (NautilusCompressDialog *self)
 {
-    GtkWidget *title;
-
-    title = gtk_label_new ("");
-    gtk_label_set_xalign (GTK_LABEL (title), 0.0);
-
-    g_object_set_data (G_OBJECT (item), "title", title);
-    gtk_list_item_set_child (item, title);
-}
-
-static gboolean
-make_opaque_iff_selected (GBinding     *binding,
-                          const GValue *from_value,
-                          GValue       *to_value,
-                          gpointer      user_data)
-{
-    GtkListItem *listitem = GTK_LIST_ITEM (user_data);
-    NautilusCompressItem *selected_item = NAUTILUS_COMPRESS_ITEM (g_value_get_object (from_value));
-    gboolean is_selected = (selected_item != NULL &&
-                            selected_item == gtk_list_item_get_item (listitem));
-    gdouble opacity = (is_selected) ? 1.0 : 0.0;
-
-    g_value_set_double (to_value, opacity);
-    return TRUE;
+    if (nautilus_filename_validator_get_has_feedback (self->validator))
+    {
+        gtk_widget_add_css_class (self->name_entry, "warning");
+    }
+    else
+    {
+        gtk_widget_remove_css_class (self->name_entry, "warning");
+    }
 }
 
 static void
-extension_dropdown_setup_item_full (GtkSignalListItemFactory *factory,
-                                    GtkListItem              *item,
-                                    gpointer                  user_data)
+on_combo_row_selected_changed (AdwComboRow *combo,
+                               GParamSpec  *spec,
+                               gpointer     user_data)
+{
+    GtkListItem *listitem = GTK_LIST_ITEM (user_data);
+    guint selected = adw_combo_row_get_selected (combo);
+    GtkWidget *child = gtk_list_item_get_child (listitem);
+    GtkWidget *checkmark = gtk_widget_get_last_child (child);
+    gdouble opacity = (selected == gtk_list_item_get_position (listitem)) ? 1.0 : 0.0;
+
+    gtk_widget_set_opacity (checkmark, opacity);
+}
+
+static void
+extension_combo_row_setup_item_full (GtkSignalListItemFactory *factory,
+                                     GtkListItem              *item,
+                                     gpointer                  user_data)
 {
-    NautilusCompressDialog *self = NAUTILUS_COMPRESS_DIALOG (user_data);
     GtkWidget *hbox, *vbox, *title, *subtitle, *checkmark;
 
     title = gtk_label_new ("");
@@ -168,11 +169,6 @@ extension_dropdown_setup_item_full (GtkSignalListItemFactory *factory,
     gtk_widget_add_css_class (subtitle, "caption");
 
     checkmark = gtk_image_new_from_icon_name ("object-select-symbolic");
-    g_object_bind_property_full (self->extension_dropdown, "selected-item",
-                                 checkmark, "opacity",
-                                 G_BINDING_SYNC_CREATE,
-                                 make_opaque_iff_selected, NULL,
-                                 item, NULL);
 
     hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
     vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 3);
@@ -187,54 +183,41 @@ extension_dropdown_setup_item_full (GtkSignalListItemFactory *factory,
     g_object_set_data (G_OBJECT (item), "subtitle", subtitle);
 
     gtk_list_item_set_child (item, hbox);
+
+    /* Checkmark will be retrieved via last_child later */
+    g_assert (gtk_widget_get_last_child (hbox) == checkmark);
 }
 
 static void
-extension_dropdown_bind (GtkSignalListItemFactory *factory,
-                         GtkListItem              *list_item,
-                         gpointer                  user_data)
+extension_combo_row_bind (GtkSignalListItemFactory *factory,
+                          GtkListItem              *list_item,
+                          gpointer                  user_data)
 {
-    NautilusCompressDialog *self = NAUTILUS_COMPRESS_DIALOG (user_data);
     GtkWidget *title, *subtitle;
     NautilusCompressItem *item;
+    NautilusCompressDialog *self = user_data;
 
     item = gtk_list_item_get_item (list_item);
 
     title = g_object_get_data (G_OBJECT (list_item), "title");
     subtitle = g_object_get_data (G_OBJECT (list_item), "subtitle");
+    g_signal_connect (self->extension_combo_row, "notify::selected",
+                      G_CALLBACK (on_combo_row_selected_changed), list_item);
+    on_combo_row_selected_changed (self->extension_combo_row, NULL, list_item);
 
-    gtk_label_set_label (GTK_LABEL (title), item->extension);
-    gtk_size_group_add_widget (self->extension_sizegroup, title);
-
-    if (item->format == NAUTILUS_COMPRESSION_ENCRYPTED_ZIP)
-    {
-        gtk_widget_add_css_class (title, "encrypted_zip");
-    }
-
-    if (subtitle)
-    {
-        gtk_label_set_label (GTK_LABEL (subtitle), item->description);
-    }
+    gtk_label_set_label (GTK_LABEL (title), item->title);
+    gtk_label_set_label (GTK_LABEL (subtitle), item->description);
 }
 
 static void
-extension_dropdown_unbind (GtkSignalListItemFactory *factory,
-                           GtkListItem              *item,
-                           gpointer                  user_data)
+extension_combo_row_unbind (GtkSignalListItemFactory *factory,
+                            GtkListItem              *list_item,
+                            gpointer                  user_data)
 {
-    NautilusCompressDialog *self = NAUTILUS_COMPRESS_DIALOG (user_data);
-    GtkWidget *title;
+    NautilusCompressDialog *self = user_data;
 
-    if (self->extension_dropdown == NULL)
-    {
-        return;
-    }
-
-    title = g_object_get_data (G_OBJECT (item), "title");
-    if (title)
-    {
-        gtk_widget_remove_css_class (title, "encrypted_zip");
-    }
+    g_signal_handlers_disconnect_by_func (self->extension_combo_row,
+                                          on_combo_row_selected_changed, list_item);
 }
 
 static gboolean
@@ -256,54 +239,54 @@ are_name_and_passphrase_ready (NautilusCompressDialog *self,
 }
 
 static void
-extension_dropdown_setup (NautilusCompressDialog *self)
+extension_combo_row_setup (NautilusCompressDialog *self)
 {
-    GtkListItemFactory *factory, *list_factory;
+    GtkListItemFactory *list_factory;
     GListStore *store;
+    g_autoptr (GtkExpression) expression = NULL;
     NautilusCompressItem *item;
     NautilusCompressionFormat format;
 
     store = g_list_store_new (NAUTILUS_TYPE_COMPRESS_ITEM);
     item = nautilus_compress_item_new (NAUTILUS_COMPRESSION_ZIP,
                                        ".zip",
+                                       _("ZIP (.zip)"),
                                        _("Compatible with all operating systems."));
     g_list_store_append (store, item);
     g_object_unref (item);
     item = nautilus_compress_item_new (NAUTILUS_COMPRESSION_ENCRYPTED_ZIP,
                                        ".zip",
-                                       _("Password protected .zip, must be installed on Windows and Mac."));
+                                       _("Encrypted ZIP (.zip)"),
+                                       _("Password-protected ZIP, must be installed on Windows and Mac."));
     g_list_store_append (store, item);
     g_object_unref (item);
     item = nautilus_compress_item_new (NAUTILUS_COMPRESSION_TAR_XZ,
                                        ".tar.xz",
+                                       _("TAR (.tar.xz)"),
                                        _("Smaller archives but Linux and Mac only."));
     g_list_store_append (store, item);
     g_object_unref (item);
     item = nautilus_compress_item_new (NAUTILUS_COMPRESSION_7ZIP,
                                        ".7z",
+                                       _("7Z (.7z)"),
                                        _("Smaller archives but must be installed on Windows and Mac."));
     g_list_store_append (store, item);
     g_object_unref (item);
 
-    factory = gtk_signal_list_item_factory_new ();
-    g_signal_connect_object (factory, "setup",
-                             G_CALLBACK (extension_dropdown_setup_item), self, 0);
-    g_signal_connect_object (factory, "bind",
-                             G_CALLBACK (extension_dropdown_bind), self, 0);
-    g_signal_connect_object (factory, "unbind",
-                             G_CALLBACK (extension_dropdown_unbind), self, 0);
-
     list_factory = gtk_signal_list_item_factory_new ();
     g_signal_connect_object (list_factory, "setup",
-                             G_CALLBACK (extension_dropdown_setup_item_full), self, 0);
+                             G_CALLBACK (extension_combo_row_setup_item_full), self, 0);
     g_signal_connect_object (list_factory, "bind",
-                             G_CALLBACK (extension_dropdown_bind), self, 0);
+                             G_CALLBACK (extension_combo_row_bind), self, 0);
     g_signal_connect_object (list_factory, "unbind",
-                             G_CALLBACK (extension_dropdown_unbind), self, 0);
+                             G_CALLBACK (extension_combo_row_unbind), self, 0);
 
-    gtk_drop_down_set_factory (GTK_DROP_DOWN (self->extension_dropdown), factory);
-    gtk_drop_down_set_list_factory (GTK_DROP_DOWN (self->extension_dropdown), list_factory);
-    gtk_drop_down_set_model (GTK_DROP_DOWN (self->extension_dropdown), G_LIST_MODEL (store));
+    expression = gtk_cclosure_expression_new (G_TYPE_STRING, NULL, 0, 0,
+                                              G_CALLBACK (nautilus_compress_item_dup_title),
+                                              NULL, NULL);
+    adw_combo_row_set_expression (self->extension_combo_row, expression);
+    adw_combo_row_set_list_factory (self->extension_combo_row, list_factory);
+    adw_combo_row_set_model (self->extension_combo_row, G_LIST_MODEL (store));
 
     format = g_settings_get_enum (nautilus_compression_preferences,
                                   NAUTILUS_PREFERENCES_DEFAULT_COMPRESSION_FORMAT);
@@ -312,7 +295,7 @@ extension_dropdown_setup (NautilusCompressDialog *self)
         item = g_list_model_get_item (G_LIST_MODEL (store), i);
         if (item->format == format)
         {
-            gtk_drop_down_set_selected (GTK_DROP_DOWN (self->extension_dropdown), i);
+            adw_combo_row_set_selected (self->extension_combo_row, i);
             set_show_passphrase (self, item->format == NAUTILUS_COMPRESSION_ENCRYPTED_ZIP);
             g_object_unref (item);
             break;
@@ -322,7 +305,6 @@ extension_dropdown_setup (NautilusCompressDialog *self)
     }
 
     g_object_unref (store);
-    g_object_unref (factory);
     g_object_unref (list_factory);
 }
 
@@ -334,19 +316,27 @@ on_name_accepted (NautilusCompressDialog *self)
 
     self->callback (name, passphrase, self->callback_data);
 
-    gtk_window_close (GTK_WINDOW (self));
+    adw_dialog_close (ADW_DIALOG (self));
 }
 
 static void
 on_name_entry_activated (NautilusCompressDialog *self)
 {
-    if (gtk_widget_is_visible (self->passphrase_entry))
+    gboolean filename_passed;
+
+    g_object_get (self->validator, "passed", &filename_passed, NULL);
+
+    if (filename_passed &&
+        !are_name_and_passphrase_ready (self,
+                                        filename_passed,
+                                        adw_combo_row_get_selected_item (self->extension_combo_row),
+                                        gtk_editable_get_text (GTK_EDITABLE (self->passphrase_entry)),
+                                        gtk_editable_get_text (GTK_EDITABLE (self->passphrase_confirm_entry))))
     {
         gtk_widget_grab_focus (self->passphrase_entry);
     }
     else
     {
-        nautilus_filename_validator_validate (self->validator);
         nautilus_filename_validator_try_accept (self->validator);
     }
 }
@@ -384,7 +374,6 @@ nautilus_compress_dialog_new (GtkWindow         *parent_window,
                               gpointer           callback_data)
 {
     NautilusCompressDialog *self = g_object_new (NAUTILUS_TYPE_COMPRESS_DIALOG,
-                                                 "transient-for", parent_window,
                                                  NULL);
 
     nautilus_filename_validator_set_containing_directory (self->validator,
@@ -397,9 +386,10 @@ nautilus_compress_dialog_new (GtkWindow         *parent_window,
     {
         gtk_editable_set_text (GTK_EDITABLE (self->name_entry), initial_name);
     }
-    gtk_widget_grab_focus (self->name_entry);
 
-    gtk_window_present (GTK_WINDOW (self));
+    adw_dialog_present (ADW_DIALOG (self), GTK_WIDGET (parent_window));
+
+    gtk_widget_grab_focus (self->name_entry);
 
     return self;
 }
@@ -410,9 +400,9 @@ nautilus_compress_dialog_init (NautilusCompressDialog *self)
     g_type_ensure (NAUTILUS_TYPE_FILENAME_VALIDATOR);
     gtk_widget_init_template (GTK_WIDGET (self));
 
-    g_signal_handlers_block_by_func (self->extension_dropdown, update_selected_format, self);
-    extension_dropdown_setup (self);
-    g_signal_handlers_unblock_by_func (self->extension_dropdown, update_selected_format, self);
+    g_signal_handlers_block_by_func (self->extension_combo_row, update_selected_format, self);
+    extension_combo_row_setup (self);
+    g_signal_handlers_unblock_by_func (self->extension_combo_row, update_selected_format, self);
 }
 
 static void
@@ -441,18 +431,17 @@ nautilus_compress_dialog_class_init (NautilusCompressDialogClass *klass)
     gtk_widget_class_set_template_from_resource (widget_class,
                                                  "/org/gnome/nautilus/ui/nautilus-compress-dialog.ui");
 
-    gtk_widget_class_bind_template_child (widget_class, NautilusCompressDialog, extension_dropdown);
-    gtk_widget_class_bind_template_child (widget_class, NautilusCompressDialog, extension_sizegroup);
+    gtk_widget_class_bind_template_child (widget_class, NautilusCompressDialog, extension_combo_row);
     gtk_widget_class_bind_template_child (widget_class, NautilusCompressDialog, name_entry);
     gtk_widget_class_bind_template_child (widget_class, NautilusCompressDialog, passphrase_confirm_entry);
-    gtk_widget_class_bind_template_child (widget_class, NautilusCompressDialog, passphrase_confirm_label);
     gtk_widget_class_bind_template_child (widget_class, NautilusCompressDialog, passphrase_entry);
-    gtk_widget_class_bind_template_child (widget_class, NautilusCompressDialog, passphrase_label);
+    gtk_widget_class_bind_template_child (widget_class, NautilusCompressDialog, passphrase_group);
     gtk_widget_class_bind_template_child (widget_class, NautilusCompressDialog, validator);
 
     gtk_widget_class_bind_template_callback (widget_class, update_selected_format);
     gtk_widget_class_bind_template_callback (widget_class, on_name_accepted);
     gtk_widget_class_bind_template_callback (widget_class, on_name_entry_activated);
+    gtk_widget_class_bind_template_callback (widget_class, on_feedback_changed);
     gtk_widget_class_bind_template_callback (widget_class, nautilus_filename_validator_try_accept);
     gtk_widget_class_bind_template_callback (widget_class, nautilus_filename_validator_validate);
     gtk_widget_class_bind_template_callback (widget_class, maybe_append_extension);
