@@ -29,12 +29,14 @@
 #include "nautilus-enums.h"
 #include "nautilus-enum-types.h"
 #include "nautilus-file.h"
+#include "nautilus-files-view.h"
 #include "nautilus-file-utilities.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-icon-names.h"
 #include "nautilus-scheme.h"
 #include "nautilus-trash-monitor.h"
 #include "nautilus-ui-utilities.h"
+#include "nautilus-window-slot.h"
 
 #ifdef GDK_WINDOWING_X11
 #include <gdk/x11/gdkx.h>
@@ -361,6 +363,7 @@ nautilus_path_bar_finalize (GObject *object)
 
     self = NAUTILUS_PATH_BAR (object);
 
+    g_clear_object (&self->current_path);
     g_clear_object (&self->current_view_menu);
     g_clear_object (&self->extensions_section);
     g_clear_object (&self->templates_submenu);
@@ -730,7 +733,7 @@ on_click_gesture_pressed (GtkGestureClick *gesture,
             else
             {
                 /* GtkButton will claim the primary button presses and emit the
-                 * "clicked" signal. Handle it in the singal callback, not here.
+                 * "clicked" signal. Handle it in the signal callback, not here.
                  */
                 return;
             }
@@ -766,7 +769,7 @@ switch_location (ButtonData *button_data)
     g_return_if_fail (self->slot != NULL);
 
     nautilus_window_slot_open_location_full (self->slot,
-                                             location, NAUTILUS_OPEN_FLAG_DONT_MAKE_ACTIVE,
+                                             location,
                                              NULL);
 }
 
@@ -881,7 +884,7 @@ on_drag_drop (GtkDropTarget *target,
     }
 
     target_location = nautilus_file_get_location (button_data->file);
-    target_view = NAUTILUS_FILES_VIEW (nautilus_window_slot_get_current_view (self->slot));
+    target_view = nautilus_window_slot_get_current_view (self->slot);
     action = gdk_drop_get_actions (gtk_drop_target_get_current_drop (target));
 
     #ifdef GDK_WINDOWING_X11
