@@ -21,12 +21,9 @@
 
 #pragma once
 
-#include "nautilus-directory.h"
+#include <glib-object.h>
 
 #define NAUTILUS_TYPE_SEARCH_ENGINE_MODEL (nautilus_search_engine_model_get_type ())
 G_DECLARE_FINAL_TYPE (NautilusSearchEngineModel, nautilus_search_engine_model, NAUTILUS, SEARCH_ENGINE_MODEL, GObject)
 
 NautilusSearchEngineModel* nautilus_search_engine_model_new       (void);
-void                       nautilus_search_engine_model_set_model (NautilusSearchEngineModel *model,
-								   NautilusDirectory         *directory);
-NautilusDirectory *        nautilus_search_engine_model_get_model (NautilusSearchEngineModel *model);
